/***
*
*    Copyright (c) 1996-2002, Valve LLC. All rights reserved.
*
*    This product contains software technology licensed from Id
*    Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc.
*    All Rights Reserved.
*
*    Compiling or running this code may violate patents in certain countries.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   Valve LLC.  All other use, distribution, or modification is prohibited
*   without written permission from Valve LLC.
*
****/

#ifndef CBASEENTITY__H
#define CBASEENTITY__H

#define MAX_ITEM_TYPES	6
#define MAX_AMMO_TYPES	32
#define MAX_AMMO_SLOTS	32

typedef struct EHANDLE_t EHANDLE;
typedef struct EHANDLE _EHANDLE;

typedef struct EHANDLE
{
	edict_t*	m_pent;
	int		m_serialnumber;
} EHANDLE_t;

typedef struct CBaseEntity_t CBaseEntity;
typedef struct CBaseEntity_s
{
	_vt_CBaseEntity*		vtable;
	#include "./offsets_cbaseentity.h"

} CBaseEntity_t;

typedef struct CBaseSpectator_t CBaseSpectator;
typedef struct CBaseSpectator_s
{
	_vt_CBaseSpectator*		vtable;
	#include "./offsets_cbaseentity.h"
	
	float		m_flFanFriction;		// 34
	float		m_flAttenuation;		// 35
	float		m_flVolume;			// 36
	float		m_pitch;			// 37
	int		m_sounds;			// 38
	
} CBaseSpectator_t;
	
typedef struct CFuncRotating_t CFuncRotating;
typedef struct CFuncRotating_s
{
	_vt_CFuncRotating*		vtable;
	#include "./offsets_cbaseentity.h"

	float		m_flFanFriction;		// 34
	float		m_flAttenuation;		// 35
	float		m_flVolume;			// 36
	float		m_pitch;			// 37
	int		m_sounds;			// 38

} CFuncRotating_t;

typedef struct CPendulum_t CPendulum;
typedef struct CPendulum_s
{
	_vt_CPendulum*		vtable;
	#include "./offsets_cbaseentity.h"

	float		m_accel;			// 34
	float		m_distance;			// 35
	float		m_time;				// 36
	float		m_damp;				// 37
	float		m_maxSpeed;			// 38
	float		m_dampSpeed;			// 39
	vec3_t		m_center;			// 40
	vec3_t		m_start;			// 43

} CPendulum_t;

typedef struct CEnvSpark_t CEnvSpark;
typedef struct CEnvSpark_s
{
	_vt_CEnvSpark*		vtable;
	#include "./offsets_cbaseentity.h"

	float		m_flDelay;

} CEnvSpark_t;

typedef struct CBubbling_t CBubbling;
typedef struct CBubbling_s
{
	_vt_CBubbling*		vtable;
	#include "./offsets_cbaseentity.h"

	int		m_density;			// 34
	int		m_frequency;			// 35
	int		m_bubbleModel;			// 36 // not found...
	int		m_state;			// 37

} CBubbling_t;

typedef struct CCyclerSprite_t CCyclerSprite;
typedef struct CCyclerSprite_s
{
	_vt_CCyclerSprite*		vtable;
	#include "./offsets_cbaseentity.h"

	int		m_animate;			// 34
	float		m_lastTime;			// 35
	float		m_maxFrame;			// 36

} CCyclerSprite_t;

typedef struct CPlatTrigger_t CPlatTrigger;
typedef struct CPlatTrigger_s
{
	_vt_CPlatTrigger*		vtable;
	#include "./offsets_cbaseentity.h"

	CBaseEntity*	m_pPlatform;

} CPlatTrigger_t;

typedef struct CGib_t CGib;
typedef struct CGib_s
{
	_vt_CGib*		vtable;
	#include "./offsets_cbaseentity.h"

	int		m_bloodColor;			// 34
	int		m_cBloodDecals;			// 35
	int		m_material;			// 36
	float		m_lifeTime;			// 37

} CGib_t;

typedef struct CAmbientGeneric_t CAmbientGeneric;
typedef struct CAmbientGeneric_s
{
	_vt_CAmbientGeneric*		vtable;
	#include "./offsets_cbaseentity.h"

	float		m_flAttenuation;			// attenuation value // 34
	dynpitchvol_t		m_dpv;				// 35
	
	BOOL		m_fActive;				// only TRUE when the entity is playing a looping sound // 60 //OK
	BOOL		m_fLooping;				// TRUE when the sound played will loop // 61 // OK

} CAmbientGeneric_t;

typedef struct CSpeaker_t CSpeaker;
typedef struct CSpeaker_s
{
	_vt_CSpeaker*		vtable;
	#include "./offsets_cbaseentity.h"

	int		m_preset;				// preset number // 34

} CSpeaker_t;

typedef struct CFuncTrackTrain_t CFuncTrackTrain;
typedef struct CFuncTrackTrain_s
{
	_vt_CFuncTrackTrain*		vtable;
	#include "./offsets_cbaseentity.h"

	CBaseEntity*	m_ppath;				// 34
	float		m_length;
	float		m_height;
	float		m_speed;
	float		m_dir;
	float		m_startSpeed;
	Vector		m_controlMins;				// 40
	Vector		m_controlMaxs;
	int		m_soundPlaying;
	int		m_sounds;
	float		m_flVolume;
	float		m_flBank;
	float		m_oldSpeed;				// 50
	
	//private:
	unsigned short m_usAdjustPitch;

} CFuncTrackTrain_t;

typedef struct CFrictionModifier_t CFrictionModifier;
typedef struct CFrictionModifier_s
{
	_vt_CFrictionModifier*		vtable;
	#include "./offsets_cbaseentity.h"	

	float		m_frictionFraction;			// Sorry, couldn't resist this name :) // 34

} CFrictionModifier_t;

typedef struct CWeaponBox_t CWeaponBox;
typedef struct CWeaponBox_s
{
	_vt_CWeaponBox*		vtable;
	#include "./offsets_cbaseentity.h"

	CBasePlayerItem*	m_rgpPlayerItems[MAX_ITEM_TYPES];	// one slot for each // 34
	int		m_rgiszAmmo[MAX_AMMO_SLOTS];		// ammo names // 40
	int		m_rgAmmo[MAX_AMMO_SLOTS];		// ammo quantities // 72
	int		m_cAmmoTypes;				// how many ammo types packed inside this box (if packed by a level designer) // 104

} CWeaponBox_t;

typedef struct CWShield_t CWShield;
typedef struct CWShield_s
{
	_vt_CWShield*		vtable;
	#include "./offsets_cbaseentity.h"

	EHANDLE	m_hLastOwner;
	float		m_flNextPickupTime;

} CWShield_t;

typedef struct CFuncTankControls_t CFuncTankControls;
typedef struct CFuncTankControls_s
{
	_vt_CFuncTankControls*		vtable;
	#include "./offsets_cbaseentity.h"

	CFuncTank*	m_pTank;			// 34

} CFuncTankControls_t;

typedef struct CItemSoda_t CItemSoda;
typedef struct CItemSoda_s
{
	_vt_CItemSoda*		vtable;
	#include "./offsets_cbaseentity.h"

} CItemSoda_t;

typedef struct CWorld_t CWorld;
typedef struct CWorld_s
{
	_vt_CWorld*		vtable;
	#include "./offsets_cbaseentity.h"

} CWorld_t;

typedef struct CDecal_t CDecal;
typedef struct CDecal_s
{
	_vt_CDecal*		vtable;
	#include "./offsets_cbaseentity.h"

} CDecal_t;

typedef struct CRenderFxManager_t CRenderFxManager;
typedef struct CRenderFxManager_s
{
	_vt_CRenderFxManager*		vtable;
	#include "./offsets_cbaseentity.h"
	
} CRenderFxManager_t;

typedef struct CShower_t CShower;
typedef struct CShower_s
{
	_vt_CShower*		vtable;
	#include "./offsets_cbaseentity.h"
	
} CShower_t;

typedef struct CSprayCan_t CSprayCan;
typedef struct CSprayCan_s
{
	_vt_CSprayCan*		vtable;
	#include "./offsets_cbaseentity.h"
	
} CSprayCan_t;

typedef struct CNullEntity_t CNullEntity;
typedef struct CNullEntity_s
{
	_vt_CNullEntity*		vtable;
	#include "./offsets_cbaseentity.h"
	
} CNullEntity_t;

typedef struct CBloodSplat_t CBloodSplat;
typedef struct CBloodSplat_s
{
	_vt_CBloodSplat*		vtable;
	#include "./offsets_cbaseentity.h"
	
} CBloodSplat_t;

typedef struct CButtonTarget_t CButtonTarget;
typedef struct CButtonTarget_s
{
	_vt_CButtonTarget*		vtable;
	#include "./offsets_cbaseentity.h"
	
} CButtonTarget_t;

typedef struct CCorpse_t CCorpse;
typedef struct CCorpse_s
{
	_vt_CCorpse*		vtable;
	#include "./offsets_cbaseentity.h"
	
} CCorpse_t;

typedef struct CFuncTrainControls_t CFuncTrainControls;
typedef struct CFuncTrainControls_s
{
	_vt_CFuncTrainControls*		vtable;
	#include "./offsets_cbaseentity.h"
	
} CFuncTrainControls_t;

typedef struct CFuncVehicleControls_t CFuncVehicleControls;
typedef struct CFuncVehicleControls_s
{
	_vt_CFuncVehicleControls*		vtable;
	#include "./offsets_cbaseentity.h"
	
} CFuncVehicleControls_t;

typedef struct CWorldItem_t CWorldItem;
typedef struct CWorldItem_s
{
	_vt_CWorldItem*		vtable;
	#include "./offsets_cbaseentity.h"

	int		_iType;

} CWorldItem_t;

typedef struct CArmoury_t CArmoury;
typedef struct CArmoury_s
{
	_vt_CArmoury*		vtable;
	#include "./offsets_cbaseentity.h"

	int		m_iItem;		// weapon ID defined in fgd
	int		m_iCount;		// item count before entity desappear
	int		m_iSaveCount;
	bool		m_bRestoreCount;
	
} CArmoury_t;

typedef struct CBaseGrenCatch_t CBaseGrenCatch;
typedef struct CBaseGrenCatch_s
{
	_vt_CBaseGrenCatch*		vtable;
	#include "./offsets_cbaseentity.h"
	
	int		m_NeedGrenadeType;              // offset 38 (152) - size 1
	char*		sTriggerOnGrenade;              // offset 39 (156) - size 1
	char*		sDisableOnGrenade;              // offset 40 (160) - size 1
	bool		m_fSmokeTouching;               // offset 41 (164) - size 1
	bool		m_fFlashTouched;                // offset 41 (165) - size 1
	bool		_dummy_41_2;
	bool		_dummy_41_3;
	
} CBaseGrenCatch_t;

typedef struct CClientFog_t CClientFog;
typedef struct CClientFog_s
{
	_vt_CClientFog*		vtable;
	#include "./offsets_cbaseentity.h"

	int		_dummy_39;
	int		_dummy_40;
	float		m_flDensity;

} CClientFog_t;

typedef struct CFuncVehicle_t CFuncVehicle;
typedef struct CFuncVehicle_s
{
	_vt_CFuncVehicle*		vtable;
	#include "./offsets_cbaseentity.h"
	
	CBaseEntity*    m_ppath;                // offset 38 (152) - size 1
	float   m_flLength;               // offset 39 (156) - size 1
	float	m_flWidth;
	float   m_flHeight;               // offset 41 (164) - size 1
	float   m_flSpeed;                // offset 42 (168) - size 1
	float   m_flDir;          // offset 43 (172) - size 1
	float   m_flStartSpeed;           // offset 44 (176) - size 1
	Vector  m_flControlMins;          // offset 45 (180) - size 1
	Vector  m_flControlMaxs;          // offset 48 (192) - size 1
	int		m_fSoundPlaying;
	int     m_sounds;               // offset 52 (208) - size 1
	int		m_iAcceleration;
	float   m_flVolume;             // offset 54 (216) - size 1
	float   m_flBank;               // offset 55 (220) - size 1
	float   m_oldSpeed;             // offset 56 (224) - size 1
	int		_dummy_57[32];
	CBasePlayer*	m_pPlayer;
	unsigned short m_usEvent;
/*
	57[0] = flag (something about traceline, sometimes checked aginst -8)
	57[1] = float timebased (maybe m_flNextTrace)
	57[2] = float timebased (maybe m_flNextUse)
	57[3] = float timebased
	57[6] = float timebased (mayba m_flLastUse)
	57[7] = float 
	57[8-9-10] = Vector
	57[11-12-13] = Vector
	57[14-15-16] = Vector
	57[17-18-19] = Vector
	57[20-21-22] = Vector
	57[23-24-25] = Vector
	57[26-27-28] = Vector
	57[29-30-31] = Vector
	57[32] =  CBasePlayer*m_pPlayer
	57[33][0] = unsigned short (events/vehicle.sc)
	57[33][1] = unsigned short 
	
*/
	
} CFuncVehicle_t;
	
typedef struct CFuncWeaponCheck_t CFuncWeaponCheck;
typedef struct CFuncWeaponCheck_s
{
	_vt_CFuncWeaponCheck*		vtable;
	#include "./offsets_cbaseentity.h"

	char*   sTriggerWithItems;              // offset 38 (152) - size 1
	char*   sTriggerNoItems;                // offset 39 (156) - size 1
	char*   sMaster;                // offset 40 (160) - size 1
	char*   sItemName[32];          // offset 41 (164) - size 32
	int     iItemCount;             // offset 73 (292) - size 1
	int     iAnyWeapon;             // offset 74 (296) - size 1	
	
} CFuncWeaponCheck_t;

typedef struct CSound
{
	Vector		m_vecOrigin;    // sound's location in space
	int		m_iType;                // what type of sound this is
	int		m_iVolume;              // how loud the sound is
	float		m_flExpireTime; // when the sound should be purged from the list
	int		m_iNext;                // index of next sound in this list ( Active or Free )
	int		m_iNextAudible; // temporary link that monsters use to build a list of audible sounds

} CSound_t;

typedef struct CSoundEnt_t CSoundEnt;
typedef struct CSoundEnt_s
{
	_vt_CSoundEnt*		vtable;
	#include "./offsets_cbaseentity.h"

	int		m_iFreeSound;   // index of the first sound in the free sound list
	int		m_iActiveSound; // indes of the first sound in the active sound list
	int		m_cLastActiveSounds; // keeps track of the number of active sounds at the last update. (for diagnostic work)
	BOOL		m_fShowReport; // if true, dump information about free/active sounds.
	
	CSound		m_SoundPool[64];
} CSoundEnt_t;
	
#endif	// CBASEENTITY__H

