// IDA HELPER TOOL
//
//
// - Set prototype functions based no mangled symbols
//		custom types should be exist on IDA before running this script
//
//

#include <idc.idc>

#define VERBOSE 1

#define VAC 1

static main(void)
{
	ProcFunctions(".text");
	ProcFunctions(".plt");
	ProcFunctions(".data");
}

static ProcFunctions(segname)
{
	auto start, end, ea;
	auto i, ret;
	auto ord, name;
	auto ok, err, errtype;

	start = SegStart(GetSeg(segname));
	end = SegEnd(start);
	ea = start - 1;
	while ((ea = NextFunction(ea)) && ea >= start && ea <= end)
	{
		name = GetFunctionName(ea);

		if(strlen(name) > 0 && strlen(GetType(ea)) > 5)
		{
			if(Demangle(name, 0))
				Message("%s|o|%s %s\n", name, GetRetType(ea), FixClassFunc(Demangle(name, 0)));
			else
				Message("%s|o|%s\n", name, FixCDECL(GetType(ea)));
		}
	}
}

static FixClassFunc(name)
{
	auto i, ret;
	
	i = strstr(name, "::");
	
	if(i > 0)
	{
		name[i] = "_";
		name[i+1] = "_";
	}
	return name;	
}

static FixCDECL(name)
{
	auto i, ret;
	
	i = strstr(name, "__cdecl(");
	
	if(i > 0)
	{
		name[i] = " ";
		name[i+1] = " ";
	}
	return name;	
}

static GetRetType(ea)
{
	auto i, ret, stype;
	
	stype = GetType(ea);
	
	for(i = 0; i < strlen(stype); i++)
	{
		if(stype[i] == " " || stype[i] == "(")
		{
			ret = substr(stype, 0, i);
			return ret;
		}
	}
	return ret;
}

static GetSeg(segname)
{
	auto ea;
	ea = 0;
	while((ea = NextSeg(ea)))
	{
		if(SegName(ea) == segname)
			return ea;
	}
	return 0;
}

static GetClass(symbol)
{
	auto i, myclass;

	for(i = 0; i < strlen(symbol); i++)
	{
		if(symbol[i] == ":" && symbol[i+1] == ":")
		{
			myclass = substr(symbol, 0, i);
			return myclass;
		}
	}
	return myclass;
}


static InVTBL(obj)
{
	auto xea, plt;
	
	for(xea=DfirstB(obj); xea != BADADDR; xea=DnextB(obj, xea))
	{
		if(strstr(Demangle(Name(xea), 0), "vtbl") >= 0)
		{
			return 1;
		}
	}
	return 0;
}