// IDA HELPER TOOL
//
//
// - Set prototype functions based no mangled symbols
//		custom types should be exist on IDA before running this script
//
//

#include <idc.idc>

#define VERBOSE 1

#define VAC 1

static main(void)
{
	ProcVtablesList(".data", 0);
	ProcVtablesList(".data", 1);
	ProcVtablesList(".data", 2);
}

static ProcVtablesList(segname, stage)
{
	auto start, end, ea;
	auto i, ret;
	auto ord, name;
	auto ok, err, errtype;

	start = SegStart(GetSeg(segname));
	end = SegEnd(start);
	ea = start;
	ok = 0;
	while (ea >= start && ea <= end && ok < 1000000)
	{
		name = Name(ea);

		if(strlen(name) > 0	&& strstr(name, "__vt_") >= 0 && strstr(name, "_ptr") < 0)
		{
				ret = SetupVtable(ea, stage);
				ok++;
		}
		
		ea = NextAddr(ea);
		
	}

	if(VERBOSE)
	{
		Message("|-------------------------------------------------------------------------------------------------|\n");
		Message("[IDA_Helper] Symbols (%s) has been processed successfully.\n", segname);
		Message("[IDA_Helper] Processing result [ok:%i] [failed:%i] [type error:%i]\n", ok, err, errtype);
		Message("|-------------------------------------------------------------------------------------------------|\n");
	}
}

static SetupVtable(ea, stage)
{
	auto i, cclass, id, eaf, tmp, idx;
	auto vclass, pclass, sclass, demangled, info;
	info = sprintf("___tf%s", substr(Name(ea), 5, -1));
	cclass = GetClass(Demangle(Name(ea), GetLongPrm(INF_LONG_DN)));
	vclass = "_vt_" + cclass;
	pclass = cclass + "_ptr";
	sclass = cclass + "_s";

//	Message("VTBL Stage %i: %s %s\n", stage, Name(ea), cclass);
	ea = ea + 4;
	idx = 0;
	
	if(stage == 0)
	{
//		Message("Hierarchy Call: %x\n", ea);
		ProcHierarchy(Dword(ea), cclass);
	}
	else if(stage == 1)
	{
		Message("typedef struct %s %s;\n", vclass, vclass);
	}
	else if(stage == 2)
	{
		Message("struct %s {\n", vclass);
		while((eaf = Dword(ea)) > 0)
		{
			idx++;
			demangled = Demangle(Name(eaf), GetLongPrm(INF_LONG_DN));
			if(strstr(Name(eaf), "__pure_virtual") == 0)
			{
				Message("\tvoid (* __pure_virtual_off%i)(void);\n", idx - 1);
			}
			else if((demangled == 0) || (strstr(demangled, "<") != -1))
			{
				Message("\tvoid (* %s%i)(void);\n", Name(eaf), idx - 1);
			}
			else if(strstr(Name(eaf), "__pure_virtual"))
			{
				Message("\tvoid (* %s; // offset %i\n", FixClass(demangled), idx - 1);
			}

			ea = ea + 4;
		}
		Message("};\n\n");

	}
}

static FixClass(symbol)
{
	auto i, myclass, argpos, newsym;
	
	for(i = 0; i < (strlen(symbol) - 1); i++)
	{
		if(symbol[i] == ":" && symbol[i+1] == ":")
		{
			myclass = GetClass(symbol);
			symbol[i] = "_";
			symbol[i+1] = "_";
		}
		

		if(symbol[i] == "(")
		{
			argpos = i+1;
		}
		
		if(symbol[i] == ")")
		{
			symbol[i+1] = ";";
		}
	}
	
	if(argpos && strlen(myclass) > 1)
	{
		newsym = substr(symbol, 0, argpos-1) +")(" + myclass + " *this, " + substr(symbol, argpos, 255);
	 	symbol = newsym;
	}
	return FixVoid(symbol);
}

static FixVoid(symbol)
{
	auto i;

	while((i = strstr(symbol,  ", void")) > 1)
	{
		symbol[i] = " ";
		symbol[i+1] = " ";
		symbol[i+2] = " ";
		symbol[i+3] = " ";
		symbol[i+4] = " ";
		symbol[i+5] = " ";
	}
	
	while((i = strstr(symbol,  ";onst")) > 1)
	{
		symbol[i+1] = " ";
		symbol[i+2] = " ";
		symbol[i+3] = " ";
		symbol[i+4] = " ";
		symbol[i] = " ";
	}
	return symbol;
}

static GetClass(symbol)
{
	auto i, myclass;

	for(i = 0; i < strlen(symbol); i++)
	{
		if(symbol[i] == ":" && symbol[i+1] == ":")
		{
			myclass = substr(symbol, 0, i);
			return myclass;
		}
	}
	return myclass;
}

static ProcHierarchy(obj, cclass)
{
	auto xea, ent, plt, instance2;
	
	plt = RfirstB(obj);
	
	for(xea=RfirstB(plt); xea != BADADDR; xea=RnextB(plt, xea))
	{
		if(strstr(GetFunctionName(xea), "__tf") == 0)
		{
			instance2 = sprintf("__%s", substr(GetFunctionName(xea), 4, -1));
			Message("\ttypedef struct %s %s;\n", cclass, GetClass(Demangle(instance2, 1)));
		}
	}
}


