/***
*
*    Copyright (c) 1996-2002, Valve LLC. All rights reserved.
*
*    This product contains software technology licensed from Id
*    Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc.
*    All Rights Reserved.
*
*    Compiling or running this code may violate patents in certain countries.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   Valve LLC.  All other use, distribution, or modification is prohibited
*   without written permission from Valve LLC.
*
****/

typedef vec3_t Vector;
typedef void * CSave;
typedef void * CRestore;
typedef int BOOL;

typedef enum { GLOBAL_OFF = 0, GLOBAL_ON = 1, GLOBAL_DEAD = 2 } GLOBALESTATE;
typedef struct globalentity_s globalentity_t;

struct globalentity_s
{
        char                    name[64];
        char                    levelName[32];
        GLOBALESTATE    state;
        globalentity_t  *pNext;
};

typedef struct locksounds                       // sounds that doors and buttons make when locked/unlocked
{
        string_t        sLockedSound;           // sound a door makes when it's locked
        string_t        sLockedSentence;        // sentence group played when door is locked
        string_t        sUnlockedSound;         // sound a door makes when it's unlocked
        string_t        sUnlockedSentence;      // sentence group played when door is unlocked

        int             iLockedSentence;                // which sentence in sentence group to play next
        int             iUnlockedSentence;              // which sentence in sentence group to play next

        float   flwaitSound;                    // time delay between playing consecutive 'locked/unlocked' sounds
        float   flwaitSentence;                 // time delay between playing consecutive sentences
        BYTE    bEOFLocked;                             // true if hit end of list of locked sentences
        BYTE    bEOFUnlocked;                   // true if hit end of list of unlocked sentences
} locksound_t;

typedef struct dynpitchvol
{
        // NOTE: do not change the order of these parameters
        // NOTE: unless you also change order of rgdpvpreset array elements!
        int preset;

        int pitchrun;           // pitch shift % when sound is running 0 - 255
        int pitchstart;         // pitch shift % when sound stops or starts 0 - 255
        int spinup;                     // spinup time 0 - 100
        int spindown;           // spindown time 0 - 100

        int volrun;                     // volume change % when sound is running 0 - 10
        int volstart;           // volume change % when sound stops or starts 0 - 10
        int fadein;                     // volume fade in time 0 - 100
        int fadeout;            // volume fade out time 0 - 100

        // Low Frequency Oscillator
        int     lfotype;                // 0) off 1) square 2) triangle 3) random
        int lforate;            // 0 - 1000, how fast lfo osciallates

        int lfomodpitch;        // 0-100 mod of current pitch. 0 is off.
        int lfomodvol;          // 0-100 mod of current volume. 0 is off.

        int cspinup;            // each trigger hit increments counter and spinup pitch


        int     cspincount;

        int pitch;
        int spinupsav;
        int spindownsav;
        int pitchfrac;

        int vol;
        int fadeinsav;
        int fadeoutsav;
        int volfrac;

        int     lfofrac;
        int     lfomult;


} dynpitchvol_t;


struct security_module_s { //0x18 most likely
	void * module;
	uint32 padding04;
/* uint32 padding08; //a 0x10 byte name?
	uint32 padding0C;
	uint32 padding10;
	uint32 padding14;
*/
	char buf10[0x10];
};

typedef security_module_s security_module_t;

struct areanode_s
{
        int             axis;           // -1 = leaf node
        float   dist;
        struct areanode_s       *children[2];
        link_t  trigger_edicts;
        link_t  solid_edicts;
};

typedef areanode_s areanode_t;

typedef struct
{
        vec3_t          boxmins, boxmaxs;// enclose the test object along entire move
        float           *mins, *maxs;   // size of the moving object
        vec3_t          mins2, maxs2;   // size when clipping against mosnters
        float           *start, *end;
        trace_t         trace;
        int                     type;
        edict_t         *passedict;
        qboolean        monsterclip;
} moveclip_t;


/*
 * typedef struct
{
        CBaseEntity             *pEntity;
        float                   amount;
        int                             type;
} MULTIDAMAGE;
*/

typedef struct
{
        int             iSlot;
        int             iPosition;
        const char      *pszAmmo1;      // ammo 1 type
        int             iMaxAmmo1;              // max ammo 1
        const char      *pszAmmo2;      // ammo 2 type
        int             iMaxAmmo2;              // max ammo 2
        const char      *pszName;
        int             iMaxClip;
        int             iId;
        int             iFlags;
        int             iWeight;// this value used to determine this weapon's importance in autoselection.
} ItemInfo;

typedef struct
{
        const char *pszName;
        int iId;
} AmmoInfo;

typedef struct
{
	float x;
	float y;
	float z;
} _Origin;

typedef struct
{
	int x;
	int y;
	int z;
} _iOrigin;

typedef struct
{
	int red;
	int green;
	int blue;
} _Color;

typedef struct  sv_clcfuncs_s
{
	int index;
	char * name;
	void * call;
} sv_clcfuncs_t;

typedef struct svstats_s
{
  double active;
  int count;
  double idle;
  double lactive;
  double lidle;
  int lpackets;
  int packets;
} svstats_t;

typedef struct packet_entities_s
{
  int num_entities;
  int max_entities;
  entity_state_t entities[64];
} packet_entities_t;

typedef struct  client_frame_s
{
  double senttime;
  float ping_time;
  clientdata_t *cdata;
  float frame_time;
  packet_entities_t entities;
} client_frame_t;

enum netadrtype_t
{
  NA_UNUSED = 0x0,
  NA_LOOPBACK = 0x1,
  NA_BROADCAST = 0x2,
  NA_IP = 0x3,
  NA_IPX = 0x4,
  NA_BROADCAST_IPX = 0x5,
};

typedef enum {
        NS_CLIENT,
        NS_SERVER
} netsrc_t;

typedef struct ipfilter_s
{
  unsigned long mask;
  unsigned long compare;
  unsigned long epoch;
  unsigned long minutes;
} ipfilter_t;

typedef struct netadr_s
{
  netadrtype_t type;
  unsigned long ip_addr;
  unsigned long ipx_addr[3];
} netadr_t;

typedef struct challenge_s
{
  netadr_t netaddress;
  int challenge;
  int time;
} challenge_t;

typedef struct client_t client_t;

typedef struct sizebuf_s
{
  unsigned int *name;
  int overflowflags;
  char *data;
  int maxsize;
  int cursize;
} sizebuf_t;

typedef struct flowstats_s
{
  int size;
  double time;
} flowstats_t;

typedef struct flow_s
{
  flowstats_t stats[32];
  int current;
  double nextcompute;
  float kbytespersec;
  float avgkbytespersec;
  int totalbytes;
} flow_t;


typedef struct fragbuf_s
{
  struct fragbuf_s *next;
  int bufferid;
  sizebuf_t frag_message;
  byte frag_message_buf[1400];
  qboolean isfile;
  qboolean isbuffer;
  char filename[64];
  int foffset;
  int size;
} fragbuf_t;

typedef struct fragbufwaiting_s
{
  struct fragbufwaiting_s *next;
  int fragbufcount;
  fragbuf_t *fragbufs;
} fragbufwaiting_t;

typedef struct  netchan_s
{
  netsrc_t sock;
  netadr_t remote_address;
  int qport;
  float last_received;
  float connect_time;
  double rate;
  double cleartime;
  int incoming_sequence;
  int incoming_acknowledged;
  int incoming_reliable_acknowledged;
  int incoming_reliable_sequence;
  int outgoing_sequence;
  int reliable_sequence;
  int last_reliable_sequence;
  client_t *netchan_client_TO_CHECK;
  int spawned;
  sizebuf_t message;
  char message_buf[3992];
  int reliable_length;
  char reliable_buf[3992];
  fragbufwaiting_t *waitlist[2];
  int reliable_fragment[2];
  unsigned int reliable_fragid[2];
  fragbuf_t *fragbufs[2];
  int fragbufcount[2];
  short frag_startpos[2];
  short frag_length[2];
  fragbuf_t *incomingbufs[2];
  qboolean incomingready[2];
  char incomingfilename[64];
  flow_t flow[2];
  int chanpayload[48];
  int tail;
} netchan_t;

typedef struct steam_s
{
  int steam_x;
  int steam_y;
  int steam_z;
} steam_t;

typedef struct  client_s
{
  int active;
  int spawned;
  int lastmsg;
  int connected;
  int resources;
  int ready;
  int consistency;
  netchan_t netchan;
  int chokecount;
  int delta_sequence;
  int fakeclient;
  int ishltv;
  usercmd_t lastcmd;
  double localtime;
  double delay;
  double nexttimeout;
  signed int ping;
  signed int packetloss;
  double unk_0;
  double nextping;
  double lastcmdtime;
  sizebuf_t datagram;
  char datagram_buffer[4000];
  double connection_started;
  double next_messagetime;
  double next_messageinterval;
  int send_message;
  int drop;
  client_frame_t *frames;
  event_info_t events[0x40];
  edict_t *edict;
  const edict_t *target_edict;
  int userid;
  steam_t steamid;
  int ipaddress;
  char setinfo[256];
  int sendinfo;
  int unk_1;
  char hashedcdkey[64];
  char playername[32];
  int topcolor;
  int bottomcolor;
  int unk_2;
  void *player_customization[34];
  resource_t resource_list;
  customization_t customization_list;
  int unk_3;
  customization_t *customization_ptr;
  int cl_lw;
  int cl_lc;
  int payload[65];
  int m_bLoopback;
  unsigned __int32 m_VoiceStreams[2];
  int newpayload0;
  int newpayload1;
  int newpayload2;
} client_t;

typedef struct  svs_s
{
  bool dll_initialized;
  client_t *clients;
  int maxclients;
  int maxclientslimit;
  int spawncount;
  int serverflags;
  int logtofile;
  bool logtoconsole;
  netadr_t logaddress;
  unsigned __int8 *logfile;
  char stats[104];
} svs_t;

typedef entvars_t * ppev;

#define MAX_RULE_BUFFER 1024

typedef struct mapcycle_item_s
{
        struct mapcycle_item_s *next;

        char mapname[ 32 ];
        int  minplayers, maxplayers;
        char rulebuffer[ MAX_RULE_BUFFER ];
} mapcycle_item_t;

typedef struct mapcycle_s
{
        struct mapcycle_item_s *items;
        struct mapcycle_item_s *next_item;
} mapcycle_t;

typedef struct hudtextparms_s
{
        float           x;
        float           y;
        int                     effect;
        byte            r1, g1, b1, a1;
        byte            r2, g2, b2, a2;
        float           fadeinTime;
        float           fadeoutTime;
        float           holdTime;
        float           fxTime;
        int                     channel;
} hudtextparms_t;

// PUT THIS IN ANOTHER FILE MAYBE ?????
typedef struct dynpitchvol
{
	// NOTE: do not change the order of these parameters 
	// NOTE: unless you also change order of rgdpvpreset array elements!
	int preset;
	
	int pitchrun;		// pitch shift % when sound is running 0 - 255
	int pitchstart;		// pitch shift % when sound stops or starts 0 - 255
	int spinup;			// spinup time 0 - 100
	int spindown;		// spindown time 0 - 100
	
	int volrun;			// volume change % when sound is running 0 - 10
	int volstart;		// volume change % when sound stops or starts 0 - 10
	int fadein;			// volume fade in time 0 - 100
	int fadeout;		// volume fade out time 0 - 100
	
	// Low Frequency Oscillator
	int	lfotype;		// 0) off 1) square 2) triangle 3) random
	int lforate;		// 0 - 1000, how fast lfo osciallates
	
	int lfomodpitch;	// 0-100 mod of current pitch. 0 is off.
	int lfomodvol;		// 0-100 mod of current volume. 0 is off.
	
	int cspinup;		// each trigger hit increments counter and spinup pitch
	
	
	int	cspincount;
	
	int pitch;			
	int spinupsav;
	int spindownsav;
	int pitchfrac;
	
	int vol;
	int fadeinsav;
	int fadeoutsav;
	int volfrac;
	
	int	lfofrac;
	int	lfomult;
	
	
} dynpitchvol;

typedef enum { expRandom, expDirected} Explosions;

typedef enum {
	matGlass = 0, matWood, matMetal, matFlesh, matCinderBlock, matCeilingTile,
	matComputer, matUnbreakableGlass, matRocks, matNone, matLastMaterial
} Materials;

typedef enum { USE_OFF = 0, USE_ON = 1, USE_SET = 2, USE_TOGGLE = 3 } USE_TYPE;

typedef struct locksounds					// sounds that doors and buttons make when locked/unlocked
{
	string_t	sLockedSound;				// sound a door makes when it's locked
	string_t	sLockedSentence;			// sentence group played when door is locked
	string_t	sUnlockedSound;				// sound a door makes when it's unlocked
	string_t	sUnlockedSentence;			// sentence group played when door is unlocked
	
	int		iLockedSentence;			// which sentence in sentence group to play next
	int		iUnlockedSentence;			// which sentence in sentence group to play next
	
	float		flwaitSound;				// time delay between playing consecutive 'locked/unlocked' sounds
	float		flwaitSentence;				// time delay between playing consecutive sentences
	BYTE		bEOFLocked;				// true if hit end of list of locked sentences
	BYTE		bEOFUnlocked;				// true if hit end of list of unlocked sentences
} locksound;

// This structure is sent over the net to describe a screen shake event
typedef struct
{
        unsigned short  amplitude;              // FIXED 4.12 amount of shake
        unsigned short  duration;               // FIXED 4.12 seconds duration
        unsigned short  frequency;              // FIXED 8.8 noise frequency (low frequency is a jerk,high frequency is a rumble)
} ScreenShake;

// This structure is sent over the net to describe a screen fade event
typedef struct
{
        unsigned short  duration;               // FIXED 4.12 seconds duration
        unsigned short  holdTime;               // FIXED 4.12 seconds duration until reset (fade & hold)
        short                   fadeFlags;              // flags
        byte                    r, g, b, a;             // fade to color ( max alpha )
} ScreenFade;

struct loopmsg_t
{
  byte data[4040];
  int datalen;
};

struct loopback_t
{
  loopmsg_t msgs[4];
  int get;
  int send;
};

